@@ -35,7 +35,7 @@ impl Samplable for BigInt {
         if bit_size == 0 {
             return BigInt::zero();
         }
-        let mut rng = OsRng::new().unwrap();
+        let mut rng = OsRng;
         let bytes = (bit_size - 1) / 8 + 1;
         let mut buf: Vec<u8> = vec![0; bytes];
         rng.fill_bytes(&mut buf);
