@@ -11,33 +11,30 @@ use super::traits::Commitment;
 use super::SECURITY_BITS;
 use crate::arithmetic::traits::*;
 
-use crate::elliptic::curves::traits::*;
+use crate::elliptic::curves::{Curve, Point, Scalar};
 use crate::BigInt;
 
 /// compute c = mG + rH
 /// where m is the commited value, G is the group generator,
 /// H is a random point and r is a blinding value.
 ///
-pub struct PedersenCommitment<P>(PhantomData<P>);
+pub struct PedersenCommitment<E: Curve>(PhantomData<E>);
 
-impl<P> Commitment<P> for PedersenCommitment<P>
-where
-    P: ECPoint,
-{
+impl<E: Curve> Commitment<Point<E>> for PedersenCommitment<E> {
     fn create_commitment_with_user_defined_randomness(
         message: &BigInt,
         blinding_factor: &BigInt,
-    ) -> P {
-        let g: P = ECPoint::generator();
-        let h = P::base_point2();
-        let message_scalar: P::Scalar = ECScalar::from(message);
-        let blinding_scalar: P::Scalar = ECScalar::from(blinding_factor);
+    ) -> Point<E> {
+        let g = Point::generator();
+        let h = Point::base_point2();
+        let message_scalar: Scalar<E> = Scalar::from(message);
+        let blinding_scalar: Scalar<E> = Scalar::from(blinding_factor);
         let mg = g * message_scalar;
         let rh = h * blinding_scalar;
         mg + rh
     }
 
-    fn create_commitment(message: &BigInt) -> (P, BigInt) {
+    fn create_commitment(message: &BigInt) -> (Point<E>, BigInt) {
         let blinding_factor = BigInt::sample(SECURITY_BITS);
         let com = PedersenCommitment::create_commitment_with_user_defined_randomness(
             message,
