@@ -6,3 +6,6 @@
 */
 
 pub mod feldman_vss;
+mod polynomial;
+
+pub use polynomial::Polynomial;
