@@ -15,7 +15,7 @@
 /// https://eprint.iacr.org/2017/552.pdf protocol 3.1 first 3 steps.
 use std::fmt::Debug;
 
-use derivative::Derivative;
+use digest::Digest;
 use serde::{Deserialize, Serialize};
 
 use crate::arithmetic::traits::*;
@@ -23,28 +23,25 @@ use crate::cryptographic_primitives::commitments::hash_commitment::HashCommitmen
 use crate::cryptographic_primitives::commitments::traits::Commitment;
 use crate::cryptographic_primitives::proofs::sigma_dlog::*;
 use crate::cryptographic_primitives::proofs::ProofError;
-use crate::elliptic::curves::traits::*;
+use crate::elliptic::curves::{Curve, Point, Scalar};
 use crate::BigInt;
-use zeroize::Zeroize;
 
 const SECURITY_BITS: usize = 256;
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
-pub struct EcKeyPair<P: ECPoint> {
-    pub public_share: P,
-    secret_share: P::Scalar,
+#[serde(bound = "")]
+pub struct EcKeyPair<E: Curve> {
+    pub public_share: Point<E>,
+    secret_share: Scalar<E>,
 }
 
-#[derive(Serialize, Deserialize, Derivative)]
-#[derivative(Clone(bound = "P: Clone, P::Scalar: Clone"))]
-#[derivative(Debug(bound = "P: Debug, P::Scalar: Debug"))]
-#[serde(bound(serialize = "P: Serialize, P::Scalar: Serialize"))]
-#[serde(bound(deserialize = "P: Deserialize<'de>, P::Scalar: Deserialize<'de>"))]
-pub struct CommWitness<P: ECPoint> {
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(bound = "")]
+pub struct CommWitness<E: Curve, H: Digest + Clone> {
     pub pk_commitment_blind_factor: BigInt,
     pub zk_pok_blind_factor: BigInt,
-    pub public_share: P,
-    pub d_log_proof: DLogProof<P>,
+    pub public_share: Point<E>,
+    pub d_log_proof: DLogProof<E, H>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -52,52 +49,43 @@ pub struct Party1FirstMessage {
     pub pk_commitment: BigInt,
     pub zk_pok_commitment: BigInt,
 }
-#[derive(Serialize, Deserialize, Derivative)]
-#[derivative(Clone(bound = "P: Clone, P::Scalar: Clone"))]
-#[derivative(Debug(bound = "P: Debug, P::Scalar: Debug"))]
-#[serde(bound(serialize = "P: Serialize, P::Scalar: Serialize"))]
-#[serde(bound(deserialize = "P: Deserialize<'de>, P::Scalar: Deserialize<'de>"))]
-pub struct Party2FirstMessage<P: ECPoint> {
-    pub d_log_proof: DLogProof<P>,
-    pub public_share: P,
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(bound = "")]
+pub struct Party2FirstMessage<E: Curve, H: Digest + Clone> {
+    pub d_log_proof: DLogProof<E, H>,
+    pub public_share: Point<E>,
 }
 
-#[derive(Serialize, Deserialize, Derivative)]
-#[derivative(Debug(bound = "P: Debug, P::Scalar: Debug"))]
-#[serde(bound(serialize = "P: Serialize, P::Scalar: Serialize"))]
-#[serde(bound(deserialize = "P: Deserialize<'de>, P::Scalar: Deserialize<'de>"))]
-pub struct Party1SecondMessage<P: ECPoint> {
-    pub comm_witness: CommWitness<P>,
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(bound = "")]
+pub struct Party1SecondMessage<E: Curve, H: Digest + Clone> {
+    pub comm_witness: CommWitness<E, H>,
 }
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct Party2SecondMessage {}
 
 impl Party1FirstMessage {
-    pub fn create_commitments<P>() -> (Party1FirstMessage, CommWitness<P>, EcKeyPair<P>)
-    where
-        P: ECPoint + Clone,
-        P::Scalar: Zeroize,
-    {
-        let base: P = ECPoint::generator();
+    pub fn create_commitments<E: Curve, H: Digest + Clone>(
+    ) -> (Party1FirstMessage, CommWitness<E, H>, EcKeyPair<E>) {
+        let base = Point::<E>::generator();
 
-        let secret_share: P::Scalar = ECScalar::new_random();
+        let secret_share = Scalar::random();
 
-        let public_share = base.scalar_mul(&secret_share.get_element());
+        let public_share = base * &secret_share;
 
-        let d_log_proof = DLogProof::<P>::prove(&secret_share);
+        let d_log_proof = DLogProof::prove(&secret_share);
         // we use hash based commitment
         let pk_commitment_blind_factor = BigInt::sample(SECURITY_BITS);
-        let pk_commitment = HashCommitment::create_commitment_with_user_defined_randomness(
-            &public_share.bytes_compressed_to_big_int(),
+        let pk_commitment = HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+            &BigInt::from_bytes(&public_share.to_bytes(true)),
             &pk_commitment_blind_factor,
         );
 
         let zk_pok_blind_factor = BigInt::sample(SECURITY_BITS);
-        let zk_pok_commitment = HashCommitment::create_commitment_with_user_defined_randomness(
-            &d_log_proof
-                .pk_t_rand_commitment
-                .bytes_compressed_to_big_int(),
+        let zk_pok_commitment = HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+            &BigInt::from_bytes(&d_log_proof.pk_t_rand_commitment.to_bytes(true)),
             &zk_pok_blind_factor,
         );
         let ec_key_pair = EcKeyPair {
@@ -119,29 +107,23 @@ impl Party1FirstMessage {
         )
     }
 
-    pub fn create_commitments_with_fixed_secret_share<P>(
-        secret_share: P::Scalar,
-    ) -> (Party1FirstMessage, CommWitness<P>, EcKeyPair<P>)
-    where
-        P: ECPoint + Clone,
-        P::Scalar: Zeroize + Clone,
-    {
-        let base: P = ECPoint::generator();
-        let public_share = base * secret_share.clone();
+    pub fn create_commitments_with_fixed_secret_share<E: Curve, H: Digest + Clone>(
+        secret_share: Scalar<E>,
+    ) -> (Party1FirstMessage, CommWitness<E, H>, EcKeyPair<E>) {
+        let base = Point::<E>::generator();
+        let public_share = base * &secret_share;
 
-        let d_log_proof = DLogProof::<P>::prove(&secret_share);
+        let d_log_proof = DLogProof::prove(&secret_share);
 
         let pk_commitment_blind_factor = BigInt::sample(SECURITY_BITS);
-        let pk_commitment = HashCommitment::create_commitment_with_user_defined_randomness(
-            &public_share.bytes_compressed_to_big_int(),
+        let pk_commitment = HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+            &BigInt::from_bytes(&public_share.to_bytes(true)),
             &pk_commitment_blind_factor,
         );
 
         let zk_pok_blind_factor = BigInt::sample(SECURITY_BITS);
-        let zk_pok_commitment = HashCommitment::create_commitment_with_user_defined_randomness(
-            &d_log_proof
-                .pk_t_rand_commitment
-                .bytes_compressed_to_big_int(),
+        let zk_pok_commitment = HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+            &BigInt::from_bytes(&d_log_proof.pk_t_rand_commitment.to_bytes(true)),
             &zk_pok_blind_factor,
         );
 
@@ -165,28 +147,20 @@ impl Party1FirstMessage {
     }
 }
 
-impl<P> Party1SecondMessage<P>
-where
-    P: ECPoint + Clone,
-    P::Scalar: Zeroize,
-{
+impl<E: Curve, H: Digest + Clone> Party1SecondMessage<E, H> {
     pub fn verify_and_decommit(
-        comm_witness: CommWitness<P>,
-        proof: &DLogProof<P>,
-    ) -> Result<Party1SecondMessage<P>, ProofError> {
+        comm_witness: CommWitness<E, H>,
+        proof: &DLogProof<E, H>,
+    ) -> Result<Party1SecondMessage<E, H>, ProofError> {
         DLogProof::verify(proof)?;
         Ok(Party1SecondMessage { comm_witness })
     }
 }
-impl<P> Party2FirstMessage<P>
-where
-    P: ECPoint + Clone,
-    P::Scalar: Zeroize + Clone,
-{
-    pub fn create() -> (Party2FirstMessage<P>, EcKeyPair<P>) {
-        let base: P = ECPoint::generator();
-        let secret_share: P::Scalar = ECScalar::new_random();
-        let public_share = base * secret_share.clone();
+impl<E: Curve, H: Digest + Clone> Party2FirstMessage<E, H> {
+    pub fn create() -> (Party2FirstMessage<E, H>, EcKeyPair<E>) {
+        let base = Point::generator();
+        let secret_share = Scalar::random();
+        let public_share = base * &secret_share;
         let d_log_proof = DLogProof::prove(&secret_share);
         let ec_key_pair = EcKeyPair {
             public_share: public_share.clone(),
@@ -202,10 +176,10 @@ where
     }
 
     pub fn create_with_fixed_secret_share(
-        secret_share: P::Scalar,
-    ) -> (Party2FirstMessage<P>, EcKeyPair<P>) {
-        let base: P = ECPoint::generator();
-        let public_share = base * secret_share.clone();
+        secret_share: Scalar<E>,
+    ) -> (Party2FirstMessage<E, H>, EcKeyPair<E>) {
+        let base = Point::generator();
+        let public_share = base * &secret_share;
         let d_log_proof = DLogProof::prove(&secret_share);
         let ec_key_pair = EcKeyPair {
             public_share: public_share.clone(),
@@ -222,19 +196,18 @@ where
 }
 
 impl Party2SecondMessage {
-    pub fn verify_commitments_and_dlog_proof<P>(
+    pub fn verify_commitments_and_dlog_proof<E: Curve, H: Digest + Clone>(
         party_one_first_message: &Party1FirstMessage,
-        party_one_second_message: &Party1SecondMessage<P>,
-    ) -> Result<Party2SecondMessage, ProofError>
-    where
-        P: ECPoint + Clone,
-        P::Scalar: Zeroize,
-    {
+        party_one_second_message: &Party1SecondMessage<E, H>,
+    ) -> Result<Party2SecondMessage, ProofError> {
         let party_one_pk_commitment = &party_one_first_message.pk_commitment;
         let party_one_zk_pok_commitment = &party_one_first_message.zk_pok_commitment;
         let party_one_zk_pok_blind_factor =
             &party_one_second_message.comm_witness.zk_pok_blind_factor;
         let party_one_public_share = &party_one_second_message.comm_witness.public_share;
+        if party_one_public_share.is_zero() {
+            return Err(ProofError);
+        }
         let party_one_pk_commitment_blind_factor = &party_one_second_message
             .comm_witness
             .pk_commitment_blind_factor;
@@ -242,8 +215,8 @@ impl Party2SecondMessage {
 
         let mut flag = true;
         if party_one_pk_commitment
-            != &HashCommitment::create_commitment_with_user_defined_randomness(
-                &party_one_public_share.bytes_compressed_to_big_int(),
+            != &HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+                &BigInt::from_bytes(&party_one_public_share.to_bytes(true)),
                 party_one_pk_commitment_blind_factor,
             )
         {
@@ -251,10 +224,8 @@ impl Party2SecondMessage {
         };
 
         if party_one_zk_pok_commitment
-            != &HashCommitment::create_commitment_with_user_defined_randomness(
-                &party_one_d_log_proof
-                    .pk_t_rand_commitment
-                    .bytes_compressed_to_big_int(),
+            != &HashCommitment::<H>::create_commitment_with_user_defined_randomness(
+                &BigInt::from_bytes(&party_one_d_log_proof.pk_t_rand_commitment.to_bytes(true)),
                 party_one_zk_pok_blind_factor,
             )
         {
@@ -266,27 +237,23 @@ impl Party2SecondMessage {
         Ok(Party2SecondMessage {})
     }
 }
-pub fn compute_pubkey<P>(local_share: &EcKeyPair<P>, other_share_public_share: &P) -> P
-where
-    P: ECPoint + Clone,
-    P::Scalar: Clone,
-{
-    other_share_public_share.clone() * local_share.secret_share.clone()
+pub fn compute_pubkey<E: Curve>(
+    local_share: &EcKeyPair<E>,
+    other_share_public_share: &Point<E>,
+) -> Point<E> {
+    other_share_public_share * &local_share.secret_share
 }
 
 #[cfg(test)]
 mod tests {
     use crate::cryptographic_primitives::twoparty::dh_key_exchange_variant_with_pok_comm::*;
 
-    crate::test_for_all_curves!(test_dh_key_exchange);
-    fn test_dh_key_exchange<P>()
-    where
-        P: ECPoint + Clone + Debug,
-        P::Scalar: Zeroize + Clone,
-    {
+    crate::test_for_all_curves_and_hashes!(test_dh_key_exchange);
+    fn test_dh_key_exchange<E: Curve, H: Digest + Clone>() {
         let (kg_party_one_first_message, kg_comm_witness, kg_ec_key_pair_party1) =
-            Party1FirstMessage::create_commitments::<P>();
-        let (kg_party_two_first_message, kg_ec_key_pair_party2) = Party2FirstMessage::<P>::create();
+            Party1FirstMessage::create_commitments::<E, H>();
+        let (kg_party_two_first_message, kg_ec_key_pair_party2) =
+            Party2FirstMessage::<E, H>::create();
         let kg_party_one_second_message = Party1SecondMessage::verify_and_decommit(
             kg_comm_witness,
             &kg_party_two_first_message.d_log_proof,
